@@ -19,6 +19,7 @@ use std::{
     ptr::NonNull,
 };
 
+use bitflags::bitflags;
 use libseccomp::ScmpSyscall;
 use memchr::arch::all::is_equal;
 use nix::{
@@ -193,16 +194,17 @@ pub struct stat32 {
 
 impl From<libc::stat64> for stat32 {
     #[allow(clippy::cast_possible_truncation)]
+    #[allow(clippy::unnecessary_cast)]
     fn from(stat: libc::stat64) -> Self {
         Self {
-            st_dev: stat.st_dev,
+            st_dev: stat.st_dev as u64,
             st_ino: stat.st_ino as u32,
             st_nlink: stat.st_nlink,
             st_mode: stat.st_mode,
             st_uid: stat.st_uid,
             st_gid: stat.st_gid,
             __pad0: 0,
-            st_rdev: stat.st_rdev,
+            st_rdev: stat.st_rdev as u64,
             st_size: stat.st_size as i32,
             st_blksize: stat.st_blksize,
             st_blocks: stat.st_blocks as i32,
@@ -1181,6 +1183,8 @@ pub(crate) const PF_INET6: libc::sa_family_t = libc::AF_INET6 as libc::sa_family
 pub(crate) const PF_ALG: libc::sa_family_t = libc::AF_ALG as libc::sa_family_t;
 #[allow(clippy::cast_possible_truncation)]
 pub(crate) const PF_NETLINK: libc::sa_family_t = libc::AF_NETLINK as libc::sa_family_t;
+/* From <bits/socket.h>, expect this to be updated regularly. */
+pub(crate) const PF_MAX: libc::sa_family_t = 46;
 
 /// nix' SockAddrLike.family() function does not support AF_ALG.
 /// This is a workaround that accesses the underlying sockaddr directly.
@@ -1189,19 +1193,6 @@ pub(crate) fn addr_family<T: SockaddrLike>(addr: &T) -> libc::sa_family_t {
     unsafe { (*addr.as_ptr()).sa_family }
 }
 
-/// Safe truncate64() wrapper.
-pub fn truncate64<P: ?Sized + NixPath>(pathname: &P, len: libc::off64_t) -> Result<(), Errno> {
-    // SAFETY: Neither nix nor libc has a wrapper for statx.
-    Errno::result(pathname.with_nix_path(|cstr| unsafe { libc::truncate64(cstr.as_ptr(), len) })?)
-        .map(drop)
-}
-
-/// Safe ftruncate64() wrapper.
-pub fn ftruncate64(fd: RawFd, len: libc::off64_t) -> Result<(), Errno> {
-    // SAFETY: nix does not have a ftruncate64 wrapper.
-    Errno::result(unsafe { libc::ftruncate64(fd, len) }).map(drop)
-}
-
 /// Want/got stx_mode & S_IFMT.
 pub const STATX_TYPE: libc::c_uint = 0x00000001;
 /// Want/got stx_mode & ~S_IFMT.
@@ -1655,3 +1646,23 @@ pub fn lsm_list_modules() -> Result<Vec<LsmId>, Errno> {
     }
     Ok(out)
 }
+
+// nix does not define RenameFlags for musl.
+bitflags! {
+    /// Flags for use with `renameat2`.
+    #[derive(Copy, Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+    #[repr(transparent)]
+    pub struct RenameFlags: u32 {
+        /// Don’t overwrite `new_path` of the rename;
+        /// return an error if `new_path` already exists.
+        const RENAME_NOREPLACE = 1;
+
+        /// Atomically exchange `old_path` and `new_path`.
+        /// Both paths must exist.
+        const RENAME_EXCHANGE  = 2;
+
+        /// Create a “whiteout” at the source of the rename
+        /// (for overlay/union filesystems).
+        const RENAME_WHITEOUT  = 4;
+    }
+}
