@@ -233,11 +233,10 @@ pub(crate) fn sysenter_exec(
     let restrict_32 = sandbox.flags.deny_elf32();
     let restrict_dyn = sandbox.flags.deny_elf_dynamic();
     let restrict_sta = sandbox.flags.deny_elf_static();
+    let restrict_ldd = !sandbox.flags.allow_unsafe_ldso();
     let restrict_pie = !sandbox.flags.allow_unsafe_nopie();
     let restrict_xs = !sandbox.flags.allow_unsafe_stack();
 
-    // Shared library execution depends on trace/allow_unsafe_exec:1.
-    let restrict_ldd = !sandbox.flags.allow_unsafe_exec();
     let check_linking = restrict_ldd || restrict_dyn || restrict_sta || restrict_pie || restrict_xs;
 
     let exe = match ExecutableFile::parse(&file, check_linking) {
@@ -292,8 +291,9 @@ pub(crate) fn sysenter_exec(
     {
         if !sandbox.filter_path(Capability::CAP_EXEC, path.abs()) {
             error!("ctx": "check_elf",
-                "err": "ld.so exec-indirection",
+                "err": "ld.so(8) exec-indirection prevented",
                 "pid": pid.as_raw(), "path": path.abs(),
+                "tip": "configure `trace/allow_unsafe_ldso:1'",
                 "exe": format!("{exe}"));
         }
         return Err(Errno::EACCES);
