@@ -206,9 +206,7 @@ pub(crate) fn sandbox_path(
 
         if new_action >= action {
             action = new_action;
-        }
-        if !filter && new_filter {
-            filter = true;
+            filter = new_filter;
         }
     }
 
