@@ -10,6 +10,7 @@ use std::{
     borrow::Cow,
     ffi::CString,
     net::IpAddr,
+    ops::Deref,
     os::{
         fd::{AsFd, AsRawFd, RawFd},
         unix::ffi::OsStrExt,
@@ -20,18 +21,13 @@ use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
     fcntl::OFlag,
-    sys::socket::{
-        getsockname, AddressFamily, MsgFlags, SockFlag, SockType, SockaddrLike, SockaddrStorage,
-    },
+    sys::socket::{getsockname, AddressFamily, MsgFlags, SockaddrLike, SockaddrStorage},
     NixPath,
 };
 
 use crate::{
-    compat::{
-        addr_family, fstatx, PF_ALG, PF_INET, PF_INET6, PF_NETLINK, PF_UNIX, PF_UNSPEC, STATX_INO,
-    },
-    cookie::safe_socket,
-    debug,
+    compat::{addr_family, PF_ALG, PF_INET, PF_INET6, PF_MAX, PF_NETLINK, PF_UNIX, PF_UNSPEC},
+    confine::{op2errno, op2name, scmp_arch_bits},
     fs::{fd_status_flags, file_type, safe_canonicalize, CanonicalPath, FileType, FsFlags},
     hook::UNotifyEventRequest,
     kernel::net::{
@@ -41,12 +37,10 @@ use crate::{
         getsockname::handle_getsockname,
         sendmsg::{handle_sendmmsg, handle_sendmsg},
         sendto::handle_sendto,
+        socket::handle_socket,
     },
-    log_enabled, op2errno, op2name,
     path::{XPath, XPathBuf},
-    sandbox::{Action, Capability, Flags, NetlinkFamily, SandboxGuard},
-    scmp_arch_bits,
-    syslog::LogLevel,
+    sandbox::{Action, Capability, SandboxGuard},
     warn,
 };
 
@@ -56,6 +50,7 @@ pub(crate) mod connect;
 pub(crate) mod getsockname;
 pub(crate) mod sendmsg;
 pub(crate) mod sendto;
+pub(crate) mod socket;
 
 const UNIX_PATH_MAX: usize = 108;
 
@@ -185,77 +180,21 @@ pub(crate) fn sys_sendmmsg(request: UNotifyEventRequest) -> ScmpNotifResp {
 fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
         let sandbox = request.get_sandbox();
-        let allow_safe_bind = sandbox.allow_safe_bind();
-        let allow_safe_kcapi = sandbox.allow_safe_kcapi();
-        let allow_unsupp_socket = sandbox.allow_unsupp_socket();
+        let allow_safe_bind = sandbox.flags.allow_safe_bind();
+        let allow_safe_kcapi = sandbox.flags.allow_safe_kcapi();
+        let allow_unsupp_socket = sandbox.flags.allow_unsupp_socket();
+        let restrict_oob = !sandbox.flags.allow_unsafe_oob();
+        let restrict_mkbdev = !sandbox.flags.allow_unsafe_mkbdev();
 
         let cap = match op {
             0x1 => {
                 // a. socket(2) system call.
                 // b. socketcall(2) -> socket(2) indirection.
-                // SAFETY: Limit available domains based on sandbox flags.
-                let domain = libc::c_int::try_from(args[0]).or(Err(Errno::EAFNOSUPPORT))?;
-                if !allow_unsupp_socket {
-                    match domain {
-                        libc::AF_UNIX | libc::AF_INET | libc::AF_INET6 => {}
-                        libc::AF_ALG if allow_safe_kcapi => {}
-                        libc::AF_NETLINK => {
-                            // Restrict AF_NETLINK to the allowlisted families.
-                            let netlink_family =
-                                u32::try_from(args[2]).or(Err(Errno::EAFNOSUPPORT))?;
-                            #[allow(clippy::cast_sign_loss)]
-                            if netlink_family > NetlinkFamily::max() as u32 {
-                                return Err(Errno::EAFNOSUPPORT);
-                            }
-                            let netlink_family = NetlinkFamily::from_bits(1 << netlink_family)
-                                .ok_or(Errno::EAFNOSUPPORT)?;
-                            if !sandbox.netlink_families.contains(netlink_family) {
-                                // SAFETY: Unsafe netlink family, deny.
-                                return Err(Errno::EAFNOSUPPORT);
-                            }
-                        }
-                        _ => return Err(Errno::EAFNOSUPPORT),
-                    }
-                } else if !allow_safe_kcapi && domain == libc::AF_ALG {
-                    return Err(Errno::EAFNOSUPPORT);
-                } else {
-                    // SAFETY: allow_unsupp_socket:1
-                    // Safe domain, allow.
-                }
-                let force_cloexec = sandbox.force_cloexec();
-                let force_rand_fd = sandbox.force_rand_fd();
-                drop(sandbox); // drop the read-lock before emulation.
+                let flags = *sandbox.flags;
+                let nlfam = sandbox.netlink_families;
+                drop(sandbox); // drop read-lock before emulation.
 
-                let stype = libc::c_int::try_from(args[1]).or(Err(Errno::EINVAL))?;
-                let proto = libc::c_int::try_from(args[2]).or(Err(Errno::EAFNOSUPPORT))?;
-                let cloexec = force_cloexec || (stype & libc::SOCK_CLOEXEC != 0);
-                let stype = stype | libc::SOCK_CLOEXEC;
-                let fd = safe_socket(domain, stype, proto)?;
-
-                if log_enabled!(LogLevel::Debug) {
-                    let inode = fstatx(&fd, STATX_INO)
-                        .map(|statx| statx.stx_ino)
-                        .unwrap_or(0);
-                    let domain = AddressFamily::from_i32(domain)
-                        .map(|af| format!("{af:?}"))
-                        .unwrap_or_else(|| "?".to_string());
-                    let flags = SockFlag::from_bits_retain(
-                        stype & (libc::SOCK_CLOEXEC | libc::SOCK_NONBLOCK),
-                    );
-                    let stype =
-                        SockType::try_from(stype & !(libc::SOCK_CLOEXEC | libc::SOCK_NONBLOCK))
-                            .map(|st| format!("{st:?}"))
-                            .unwrap_or_else(|_| "?".to_string());
-                    debug!("ctx": "net", "op": "create_socket",
-                        "msg": format!("created {domain} {stype} socket with inode:{inode:#x}"),
-                        "domain": domain,
-                        "type": stype,
-                        "protocol": proto,
-                        "flags": flags.bits(),
-                        "inode": inode);
-                }
-
-                return request.send_fd(fd, cloexec, force_rand_fd);
+                return handle_socket(&request, args, flags, nlfam);
             }
             0x2 | 0x6 => Capability::CAP_NET_BIND,
             _ => Capability::CAP_NET_CONNECT,
@@ -291,11 +230,25 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
             }
             0x10 => {
                 // sendmsg uses a different data structure, so we handle it in its own branch.
-                return handle_sendmsg(fd, &request, args, allow_unsupp_socket);
+                return handle_sendmsg(
+                    fd,
+                    &request,
+                    args,
+                    allow_unsupp_socket,
+                    restrict_oob,
+                    restrict_mkbdev,
+                );
             }
             0x14 => {
                 // sendmmsg uses a different data structure, so we handle it in its own branch.
-                return handle_sendmmsg(fd, &request, args, allow_unsupp_socket);
+                return handle_sendmmsg(
+                    fd,
+                    &request,
+                    args,
+                    allow_unsupp_socket,
+                    restrict_oob,
+                    restrict_mkbdev,
+                );
             }
             _ => {} // fall through.
         }
@@ -307,19 +260,8 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
         let addr_len = args[idx + 1] as libc::socklen_t;
         if addr_remote == 0 && addr_len == 0 {
             if op == 0xb {
-                // sendto
-                // SAFETY: Connection mode socket.
-                // Note, we don't hook into sendto anymore when both
-                // these arguments are NULL so we can never be here
-                // practically. However, this can still happen in
-                // theory, e.g. when the syscall is originating from a
-                // multiplexed socketcall() syscall. In this case
-                // continuing the system call here obviously would open a
-                // TOCTOU window so instead we pass a None address to
-                // handle_sendto() where we'll invoke the send() syscall
-                // on our own and return the result to the sandbox
-                // process.
-                return handle_sendto(fd, args, &request, None);
+                // sendto: Connection mode socket.
+                return handle_sendto(fd, args, &request, None, restrict_oob);
             } else {
                 return Err(Errno::EFAULT);
             }
@@ -333,7 +275,6 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
             &sandbox,
             get_addr(&request, addr_remote, addr_len)?,
             cap,
-            *sandbox.flags,
         )?;
         match addr_family(&addr) {
             PF_UNIX | PF_INET | PF_INET6 => {
@@ -361,6 +302,7 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
                 // b. Non-bind() call is not supported for AF_ALG socket.
                 return Err(Errno::EOPNOTSUPP);
             }
+            n if n >= PF_MAX => return Err(Errno::EINVAL),
             _ if allow_unsupp_socket => {
                 // SAFETY: This is also restricted at socket(2) boundary
                 // where there is no pointer-dereference in access
@@ -376,7 +318,7 @@ fn syscall_network_handler(request: UNotifyEventRequest, args: &[u64; 6], op: u8
         match op {
             0x2 => handle_bind(fd, &addr, root.as_ref(), &request, allow_safe_bind),
             0x3 => handle_connect(fd, &addr, &request, allow_safe_bind),
-            0xb => handle_sendto(fd, args, &request, Some(&addr)),
+            0xb => handle_sendto(fd, args, &request, Some(&addr), restrict_oob),
             _ => unreachable!(),
         }
     })
@@ -422,7 +364,6 @@ fn canon_addr<'a>(
     sandbox: &SandboxGuard,
     addr: SockaddrStorage,
     cap: Capability,
-    flags: Flags,
 ) -> Result<(SockaddrStorage, Option<CanonicalPath<'a>>), Errno> {
     #[allow(clippy::cast_possible_truncation)]
     if let Some(path) = addr.as_unix_addr().and_then(|a| a.path()) {
@@ -438,10 +379,10 @@ fn canon_addr<'a>(
         let null = memchr::memchr(0, path).unwrap_or(path.len());
         let path = XPathBuf::from(&path[..null]);
 
-        // If bind, the path may or may not exist depending on SO_REUSEADDR
-        // Else, the path must exist.
+        // For bind(2), the path must be missing or we return EADDRINUSE.
+        // For connect family, the path must exist or we return ENOENT.
         let fsflags = if cap == Capability::CAP_NET_BIND {
-            FsFlags::empty()
+            FsFlags::MISS_LAST
         } else {
             FsFlags::MUST_PATH
         };
@@ -451,7 +392,7 @@ fn canon_addr<'a>(
         // 1. Always resolve symlinks.
         // 2. Ensure relative UNIX socket paths match process CWD.
         let pid = request.scmpreq.pid();
-        let path = safe_canonicalize(pid, None, &path, fsflags, flags)?;
+        let path = safe_canonicalize(pid, None, &path, fsflags, Some(sandbox.deref()))?;
 
         let cstr = if path.base.is_empty() {
             // SAFETY: We open a FD to the path and then use the
@@ -775,7 +716,7 @@ pub(crate) fn sandbox_addr_inet6(
 
 /// Process a socket address of an unsupported socket family.
 pub(crate) fn sandbox_addr_notsup(sandbox: &SandboxGuard) -> Result<(), Errno> {
-    if sandbox.allow_unsupp_socket() {
+    if sandbox.flags.allow_unsupp_socket() {
         Ok(())
     } else {
         Err(Errno::EAFNOSUPPORT)
