@@ -24,15 +24,16 @@ use nix::{
 
 use crate::{
     compat::{fstatx, STATX_MODE, STATX_UID},
-    config::{ESYD_SH, MAGIC_PREFIX, NULL_FD, PROC_FILE, ROOT_FILE, SYD_EL},
+    config::{ESYD_SH, MAGIC_PREFIX, NULL_FD, PROC_FILE, ROOT_FILE, SAFE_MFD_FLAGS, SYD_EL},
     cookie::safe_openat2,
     fs::{
-        create_memfd, lock_fd, safe_open_how_magicsym, safe_open_magicsym, seal_memfd, unlock_fd,
-        CanonicalPath, FileInfo, FileType, FsFlags, MaybeFd, MFD_ALLOW_SEALING,
+        create_memfd, is_dev_kfd, lock_fd, safe_open_how_magicsym, safe_open_magicsym, seal_memfd,
+        unlock_fd, CanonicalPath, FileInfo, FileType, FsFlags, MaybeFd,
     },
     hash::aes_ctr_tmp,
     hook::{SysArg, UNotifyEventRequest},
     info, log_enabled,
+    mask::mask_proc_pid_status,
     path::{XPath, XPathBuf},
     proc::{proc_tty, proc_umask},
     sandbox::{Action, Capability},
@@ -143,7 +144,12 @@ pub(crate) fn sys_openat2(request: UNotifyEventRequest) -> ScmpNotifResp {
     // 2. Return EINVAL for invalid resolve flags.
     let rflags = match ResolveFlag::from_bits(open_how.resolve) {
         Some(rflags) if rflags.contains(ResolveFlag::RESOLVE_IN_ROOT) => {
-            return request.fail_syscall(Errno::ENOSYS)
+            let errno = if rflags.contains(ResolveFlag::RESOLVE_BENEATH) {
+                Errno::EINVAL
+            } else {
+                Errno::ENOSYS
+            };
+            return request.fail_syscall(errno);
         }
         Some(rflags) => rflags,
         None => return request.fail_syscall(Errno::EINVAL),
@@ -206,8 +212,9 @@ fn syscall_open_handler(
         let o_trunc = flags.contains(OFlag::O_TRUNC);
         let mut o_mask = o_creat || o_tmpfl;
 
-        // O_PATH|O_CREAT and O_PATH|O_TMPFILE combinations are invalid.
-        if o_path && o_mask {
+        // O_PATH|O_CREAT, O_PATH|O_TMPFILE, and O_CREAT|O_TMPFILE
+        // combinations are invalid.
+        if (o_path && o_mask) || (o_creat && o_tmpfl) {
             return Err(Errno::EINVAL);
         }
 
@@ -232,8 +239,8 @@ fn syscall_open_handler(
             caps.insert(Capability::CAP_TRUNCATE);
         }
 
-        let has_write = caps.intersects(Capability::CAP_WRSET);
-        if has_write && sandbox.force_ro_open() {
+        let can_write = caps.can_write();
+        if can_write && sandbox.flags.force_ro_open() {
             // SAFETY: Deny write attempt with trace/force_ro_open:1.
             return Err(Errno::EACCES);
         }
@@ -242,7 +249,6 @@ fn syscall_open_handler(
             crate::debug!("ctx": "open", "op": "init",
                 "cap": caps,
                 "sbc": *sandbox.state,
-                "cwr": caps.intersects(Capability::CAP_WRSET),
                 "arg": format!("{arg:?}"),
                 "flg": format!("{flags:?}"),
                 "mod": format!("{mode:?}"),
@@ -251,7 +257,6 @@ fn syscall_open_handler(
             crate::debug!("ctx": "open", "op": "init",
                 "cap": caps,
                 "sbc": *sandbox.state,
-                "cwr": caps.intersects(Capability::CAP_WRSET),
                 "arg": format!("{arg:?}"),
                 "flg": format!("{flags:?}"),
                 "mod": format!("{mode:?}"),
@@ -260,7 +265,7 @@ fn syscall_open_handler(
 
         // SAFETY: Apply deny_dotdot as necessary for open family.
         let mut arg = arg;
-        if sandbox.deny_dotdot() {
+        if sandbox.flags.deny_dotdot() {
             arg.fsflags.insert(FsFlags::NO_RESOLVE_DOTDOT);
         }
 
@@ -285,7 +290,7 @@ fn syscall_open_handler(
             }
 
             let fd = if caps.contains(Capability::CAP_READ) {
-                let fd = create_memfd(b"syd-box\0", MFD_ALLOW_SEALING)?;
+                let fd = create_memfd(b"syd-box\0", *SAFE_MFD_FLAGS)?;
                 let mut file = File::from(fd);
 
                 const FLEN: usize = MAGIC_PREFIX.len() + 3; /* .{el,sh} */
@@ -324,7 +329,7 @@ fn syscall_open_handler(
         // is_blocking: Used by interrupt handler to unblock FIFOs.
         // may_crypt: Used by Crypt sandboxing.
         let (is_dir, is_blocking, may_crypt) = if let Some(ref file_type) = path.typ {
-            if o_creat && !o_tmpfl {
+            if o_creat {
                 // Creating open with existing file:
                 // Remove Create from capabilities, and add back
                 // Truncate capability as necessary to improve
@@ -356,14 +361,13 @@ fn syscall_open_handler(
 
                     (true, false, false)
                 }
-                FileType::Blk | FileType::Unk => {
+                FileType::Mfd => {
                     // SAFETY:
-                    // 1. Do not allow access to block devices.
-                    // 2. Do not allow access to files with unknown types.
-                    // 3. Deny with ENOENT for stealth.
-                    return Err(Errno::ENOENT);
+                    // 1. We do not encrypt memory fds.
+                    // 2. We do not support interruptions on memory fd blocks.
+                    (false, false, false)
                 }
-                FileType::MagicLnk(_, _) => {
+                FileType::MagicLnk(_) => {
                     // SAFETY/TODO: Implement further restrictions on magic-links here.
                     if file_type.is_magic_dir() {
                         // Change capability from Read to Readdir.
@@ -371,7 +375,7 @@ fn syscall_open_handler(
                         caps.insert(Capability::CAP_READDIR);
                     }
 
-                    // SAFETY: This may or may not block, better safe than sorry.
+                    // This may or may not block, better safe than sorry.
                     (
                         false,
                         !(flags.contains(OFlag::O_NONBLOCK) || flags.contains(OFlag::O_NDELAY)),
@@ -379,7 +383,7 @@ fn syscall_open_handler(
                     )
                 }
                 FileType::Lnk => {
-                    // SAFETY: This may or may not block, better safe than sorry.
+                    // This may or may not block, better safe than sorry.
                     (
                         false,
                         !(flags.contains(OFlag::O_NONBLOCK) || flags.contains(OFlag::O_NDELAY)),
@@ -387,18 +391,34 @@ fn syscall_open_handler(
                     )
                 }
                 FileType::Chr | FileType::Fifo | FileType::Sock => {
-                    // SAFETY: character devices, fifos and sockets may block.
+                    // Character devices, fifos and sockets may block.
                     (
                         false,
                         !(flags.contains(OFlag::O_NONBLOCK) || flags.contains(OFlag::O_NDELAY)),
                         false,
                     )
                 }
-                FileType::Mfd => {
+                FileType::Blk => {
                     // SAFETY:
-                    // 1. We do not encrypt memory fds.
-                    // 2. We do not support interruptions on memory fd blocks.
-                    (false, false, false)
+                    // 1. Do not allow access to block devices.
+                    // 2. Restriction may be relaxed with trace/allow_unsafe_mkbdev:1.
+                    // 3. Deny with ENOENT for stealth.
+                    if !sandbox.flags.allow_unsafe_mkbdev() {
+                        return Err(Errno::ENOENT);
+                    }
+
+                    // Block devices may block.
+                    (
+                        false,
+                        !(flags.contains(OFlag::O_NONBLOCK) || flags.contains(OFlag::O_NDELAY)),
+                        false,
+                    )
+                }
+                FileType::Unk => {
+                    // SAFETY:
+                    // 1. Do not allow access to files with unknown types.
+                    // 2. Deny with ENOENT for stealth.
+                    return Err(Errno::ENOENT);
                 }
             }
         } else if !o_creat {
@@ -478,9 +498,7 @@ fn syscall_open_handler(
                     let (new_action, new_filter) = sandbox.check_path(cap, &path_check);
                     if new_action >= action {
                         action = new_action;
-                    }
-                    if !filter && new_filter {
-                        filter = true;
+                        filter = new_filter;
                     }
                 }
             } else {
@@ -491,9 +509,7 @@ fn syscall_open_handler(
                         let (new_action, new_filter) = sandbox.check_path(cap, path);
                         if new_action >= action {
                             action = new_action;
-                        }
-                        if !filter && new_filter {
-                            filter = true;
+                            filter = new_filter;
                         }
                     }
                 }
@@ -542,7 +558,7 @@ fn syscall_open_handler(
 
             // Check for append-only path and edit flags argument.
             // Temporary files can not be made append-only.
-            if has_write && sandbox.is_append(&path_check) {
+            if can_write && sandbox.is_append(&path_check) {
                 flags.insert(OFlag::O_APPEND);
                 flags.remove(OFlag::O_TRUNC);
 
@@ -552,7 +568,6 @@ fn syscall_open_handler(
                             "sys": request.syscall, "path": &path_check,
                             "flags": format!("{flags:?}"),
                             "cap": caps,
-                            "cap_write_set": Capability::CAP_WRSET,
                             "req": &request);
                 } else {
                     info!("ctx": "open", "op": "set_append_only",
@@ -560,7 +575,6 @@ fn syscall_open_handler(
                             "sys": request.syscall, "path": &path_check,
                             "flags": format!("{flags:?}"),
                             "cap": caps,
-                            "cap_write_set": Capability::CAP_WRSET,
                             "pid": request.scmpreq.pid);
                 }
             }
@@ -570,14 +584,15 @@ fn syscall_open_handler(
 
         let force_umask = sandbox.umask;
         let verbose = sandbox.verbose;
-        let unsafe_open_path = sandbox.allow_unsafe_open_path();
-        let unsafe_open_cdev = sandbox.allow_unsafe_open_cdev();
-        let restrict_create = !sandbox.allow_unsafe_create();
-        let restrict_memfd = !sandbox.allow_unsafe_memfd();
-        if sandbox.force_cloexec() {
+        let unsafe_kfd = sandbox.flags.allow_unsafe_kfd();
+        let unsafe_open_path = sandbox.flags.allow_unsafe_open_path();
+        let restrict_create = !sandbox.flags.allow_unsafe_create();
+        let filter_proc_status =
+            !sandbox.flags.allow_unsafe_proc_status() && path.abs().is_proc_pid_status();
+        if sandbox.flags.force_cloexec() {
             o_cloexec = true;
         }
-        let o_rand_fd = sandbox.force_rand_fd();
+        let o_rand_fd = sandbox.flags.force_rand_fd();
         let setup_fds = sandbox.crypt_setup();
         let crypt_tmp = if crypted { sandbox.crypt_tmp } else { None };
         drop(sandbox); // release the read-lock.
@@ -622,7 +637,7 @@ fn syscall_open_handler(
         }
 
         if let Some(file_type) = &path.typ {
-            if !matches!(file_type, FileType::Dir | FileType::MagicLnk(_, _))
+            if !matches!(file_type, FileType::Dir | FileType::MagicLnk(_))
                 && path.abs().last() == Some(b'/')
             {
                 return Err(Errno::ENOTDIR);
@@ -630,15 +645,16 @@ fn syscall_open_handler(
         }
 
         if let Some(FileType::Dir) = &path.typ {
-            if !o_tmpfl {
-                if orig_caps.can_write() {
-                    // Open for write on directory.
-                    return Err(Errno::EISDIR);
-                } else if o_creat && path.abs().last() == Some(b'/') {
-                    // Creating open on directory.
-                    return Err(Errno::EISDIR);
-                }
+            if !o_tmpfl && orig_caps.can_write() {
+                // Open for write on directory.
+                return Err(Errno::EISDIR);
+            } else if o_creat && path.abs().last() == Some(b'/') {
+                // Creating open on directory.
+                return Err(Errno::EISDIR);
             }
+        } else if o_tmpfl {
+            // O_TMPFILE open on non-directory.
+            return Err(Errno::ENOTDIR);
         }
 
         // SAFETY: Return EPERM for {/dev,/proc}/kmsg,
@@ -713,49 +729,39 @@ fn syscall_open_handler(
                     // SAFETY: Turn O_PATH flag to O_RDONLY for successful emulation.
                     flags.remove(OFlag::O_PATH);
                     flags.insert(OFlag::O_RDONLY);
-                } else if unsafe_open_cdev
+                } else if unsafe_kfd
                     && o_rdwr
                     && !o_creat
                     && !o_trunc
                     && !o_tmpfl
-                    && path.typ == Some(FileType::Chr)
+                    && is_dev_kfd(path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?)
+                        .unwrap_or(false)
                 {
                     // SAFETY:
                     //
-                    // trace/allow_unsafe_open_cdev:true
+                    // trace/allow_unsafe_kfd:true
                     //
-                    // 1. Some character devices, such as AMD GPUs,
-                    //    require per-application access to the GPU
-                    //    device, therefore opening the device in the
-                    //    Syd emulator thread and then continuing the
-                    //    subsequent ioctl(2) system calls in the
-                    //    sandbox process is going to return EBADF.
-                    //    Until, Syd has a way to fully emulate the
-                    //    ioctl(2) request space and are able to call
-                    //    ioctl(2) directly from Syd emulator threads,
-                    //    this option may be used to access such
-                    //    character devices. Note, setting this option
-                    //    opens a TOCTOU attack vector, whereby the
-                    //    sandbox process can open an arbitrary file
-                    //    instead of the character device in question!
-                    // 2. Syd does not CONTINUE the system call if at
-                    //    least one of the flags
-                    //    O_CREAT|O_TRUNC|O_TMPFILE is set in flags
-                    //    argument to limit the scope of the TOCTOU
-                    //    attack vector.
-                    //  3. Syd CONTINUEs the system call if and only if
-                    //     O_RDWR is set in the flags argument to limit
-                    //     the scope of the TOCTOU attack vector.
-                    //  4. Syd returns ENOSYS for openat2(2) rather than
-                    //     CONTINUE'ing the system call to prevent the
-                    //     "struct open_how" pointer indirection to
-                    //     bypass the restrictions applied to the flags
-                    //     argument.
-                    //  5. This option may be changed at runtime, so it
-                    //     is highly recommended to unset this option
-                    //     right after the respective character device
-                    //     is open using the syd(2) API to prevent the
-                    //     TOCTOU attack vector.
+                    // 1. AMD KFD character devices require per-application access
+                    //    to the GPU device, therefore opening the device in the
+                    //    Syd emulator thread and then continuing the subsequent ioctl(2)
+                    //    system calls in the sandbox process is going to return EBADF.
+                    //    Until, Syd has a way to fully emulate the ioctl(2) request space
+                    //    and is able to call ioctl(2) directly from Syd emulator threads,
+                    //    this option may be used to access such character devices.
+                    //    Setting this option opens a TOCTOU attack vector, whereby the
+                    //    sandbox process can open an arbitrary file instead of the character
+                    //    device in question!
+                    // 2. Syd does not CONTINUE the system call if at least one of the flags
+                    //    O_CREAT|O_TRUNC|O_TMPFILE is set in flags argument to limit the scope
+                    //    of the TOCTOU attack vector.
+                    //  3. Syd CONTINUEs the system call if and only if O_RDWR is set in the flags
+                    //     argument to limit the scope of the TOCTOU attack vector.
+                    //  4. Syd returns ENOSYS for openat2(2) rather than CONTINUE'ing the system call
+                    //     to prevent the "struct open_how" pointer indirection to bypass the restrictions
+                    //     applied to the flags argument.
+                    //  5. This option may be changed at runtime, and it is highly recommended to unset
+                    //     this option right after the respective character device is open using the syd(2)
+                    //     API to close the TOCTOU attack vector.
                     return if syscall != OpenSyscall::Openat2 {
                         // SAFETY: See above, stupid clippy.
                         unsafe { Ok(request.continue_syscall()) }
@@ -772,9 +778,8 @@ fn syscall_open_handler(
                 // 3. Add RESOLVE_NO_MAGICLINKS | RESOLVE_NO_SYMLINKS to flags.
                 // 4. Mode must be 0 if O_CREAT or O_TMPFILE is not in flags.
                 // Note, magic symbolic links are an exception here.
-                let mut resolve_flags = ResolveFlag::empty();
-                if matches!(path.typ, Some(FileType::MagicLnk(_, _))) {
-                    if flags.contains(OFlag::O_NOFOLLOW) {
+                if matches!(path.typ, Some(FileType::MagicLnk(_))) {
+                    if !path.base.is_empty() || flags.contains(OFlag::O_NOFOLLOW) {
                         // Magic symlink O_PATH fds were continued
                         // as necessary if relevant unsafe options
                         // were set. After this point, we have to
@@ -792,12 +797,8 @@ fn syscall_open_handler(
                     crypted = false;
                     flags.remove(OFlag::O_CREAT);
                     flags.remove(OFlag::O_TMPFILE);
-                } else {
-                    flags.insert(OFlag::O_NOFOLLOW);
-                    resolve_flags.insert(ResolveFlag::RESOLVE_BENEATH);
-                    resolve_flags.insert(ResolveFlag::RESOLVE_NO_MAGICLINKS);
-                    resolve_flags.insert(ResolveFlag::RESOLVE_NO_SYMLINKS);
                 }
+                flags.insert(OFlag::O_NOFOLLOW);
 
                 let fd: OwnedFd = if crypted {
                     // Handle Encryption.
@@ -811,7 +812,6 @@ fn syscall_open_handler(
                         force_umask,
                         o_cloexec,
                         o_rand_fd,
-                        restrict_memfd,
                     )? {
                         // read-only encryption.
                         fd
@@ -839,6 +839,7 @@ fn syscall_open_handler(
                     let pfd = XPathBuf::from_self_fd(fd);
 
                     // Reopen the `O_PATH` path fd with the requested flags.
+                    flags.remove(OFlag::O_CREAT);
                     flags.remove(OFlag::O_NOFOLLOW);
                     let how = safe_open_how_magicsym(flags);
 
@@ -856,37 +857,34 @@ fn syscall_open_handler(
                             .del_sys_block(req.id, matches!(result, Err(Errno::EINTR)))?;
                     }
 
-                    result?
+                    let mut fd = result?;
+
+                    // SAFETY: Filter security-sensitive proc_pid_status(5) contents.
+                    if filter_proc_status {
+                        // Open a memory fd to be sealed for write.
+                        // Memory fd is exec-sealed on Linux>=6.3.
+                        let mut mfd =
+                            create_memfd(b"syd-pps\0", *SAFE_MFD_FLAGS).map(File::from)?;
+
+                        // Filter proc_pid_status(5) into the memory fd.
+                        mask_proc_pid_status(fd, &mfd)?;
+
+                        // Rewind file offset and seal to deny further writes.
+                        mfd.rewind().or(Err(Errno::EIO))?;
+                        seal_memfd(&mfd)?;
+
+                        fd = mfd.into();
+                    }
+
+                    fd
                 } else if o_mask {
                     // Cannot be O_PATH or encrypted fd!
                     // SAFETY: If we're creating the file,
-                    // we must fork so we can apply the umask
-                    // and still honour POSIX ACLs.
+                    // we need to change per-thread umask and
+                    // still honour POSIX ACLs.
                     handle_creat(&request, &path, flags, mode, force_umask)?
                 } else {
-                    // Prepare `struct open_how`.
-                    // Mode is empty because we handle creation differently.
-                    let how = OpenHow::new().flags(flags).resolve(resolve_flags);
-
-                    // Prepare file descriptor, avoid relative pathnames.
-                    let fd = path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?;
-
-                    // SAFETY: Record blocking call so it can get invalidated.
-                    if is_blocking {
-                        request.cache.add_sys_block(req, false)?;
-                    }
-
-                    // All set, open the file.
-                    let result = safe_openat2(fd, path.base, how);
-
-                    // Remove invalidation record unless interrupted.
-                    if is_blocking {
-                        request
-                            .cache
-                            .del_sys_block(req.id, matches!(result, Err(Errno::EINTR)))?;
-                    }
-
-                    result?
+                    unreachable!("BUG: MUST_PATH returned a directory for open, report a bug!");
                 };
 
                 // File opened successfully, return to caller at one go.
@@ -957,7 +955,6 @@ fn handle_crypt(
     force_umask: Option<Mode>,
     o_cloexec: bool,
     o_rand_fd: bool,
-    restrict_memfd: bool,
 ) -> Result<Option<OwnedFd>, Errno> {
     #[allow(clippy::disallowed_methods)]
     let files = request.crypt_map.as_ref().unwrap();
@@ -1048,9 +1045,7 @@ fn handle_crypt(
     // Note, enc_fd is an OwnedFd so in the event of
     // an aes_ctr_tmp error it'll be closed and the lock
     // will be released.
-    let (fd, iv) = if let Some((fd, iv)) =
-        aes_ctr_tmp(setup_fds, &enc_fd, safe_flags, tmpdir, restrict_memfd)?
-    {
+    let (fd, iv) = if let Some((fd, iv)) = aes_ctr_tmp(setup_fds, &enc_fd, safe_flags, tmpdir)? {
         (fd, iv)
     } else {
         // SAFETY:
