@@ -1,4 +1,7 @@
+// SPDX-License-Identifier: Apache-2.0 OR MIT
+
 use std::{
+    fmt,
     fs::OpenOptions,
     io::Error,
     mem::zeroed,
@@ -16,17 +19,21 @@ use nix::{
     fcntl::{open, OFlag},
     sys::stat::Mode,
 };
+use serde::{Serialize, Serializer};
 #[cfg(test)]
 use strum::IntoEnumIterator;
 
-use crate::landlock::{
-    compat::private::OptionCompatLevelMut, uapi, Access, AddRuleError, AddRulesError, CompatError,
-    CompatLevel, CompatResult, CompatState, Compatible, HandleAccessError, HandleAccessesError,
-    PathBeneathError, PathFdError, PrivateAccess, PrivateRule, Rule, Ruleset, RulesetCreated,
-    RulesetError, TailoredCompatLevel, TryCompat, ABI,
-};
 #[cfg(test)]
 use crate::landlock::{AccessError, RulesetAttr, RulesetCreatedAttr};
+use crate::{
+    fs::{file_type, FileType},
+    landlock::{
+        compat::private::OptionCompatLevelMut, uapi, Access, AddRuleError, AddRulesError,
+        CompatError, CompatLevel, CompatResult, CompatState, Compatible, HandleAccessError,
+        HandleAccessesError, PathBeneathError, PathFdError, PrivateAccess, PrivateRule, Rule,
+        Ruleset, RulesetCreated, RulesetError, TailoredCompatLevel, TryCompat, ABI,
+    },
+};
 
 crate::landlock::access::bitflags_type! {
     /// File system access right.
@@ -111,7 +118,7 @@ impl AccessFs {
     pub fn from_read(abi: ABI) -> Self {
         match abi {
             ABI::Unsupported => AccessFs::EMPTY,
-            ABI::V1 | ABI::V2 | ABI::V3 | ABI::V4 | ABI::V5 | ABI::V6 => {
+            ABI::V1 | ABI::V2 | ABI::V3 | ABI::V4 | ABI::V5 | ABI::V6 | ABI::V7 => {
                 make_bitflags!(AccessFs::{
                     Execute
                     | ReadFile
@@ -141,7 +148,7 @@ impl AccessFs {
             }),
             ABI::V2 => Self::from_write(ABI::V1) | AccessFs::Refer,
             ABI::V3 | ABI::V4 => Self::from_write(ABI::V2) | AccessFs::Truncate,
-            ABI::V5 | ABI::V6 => Self::from_write(ABI::V4) | AccessFs::IoctlDev,
+            ABI::V5 | ABI::V6 | ABI::V7 => Self::from_write(ABI::V4) | AccessFs::IoctlDev,
         }
     }
 
@@ -157,8 +164,10 @@ fn consistent_access_fs_rw() {
         let access_all = AccessFs::from_all(abi);
         let access_read = AccessFs::from_read(abi);
         let access_write = AccessFs::from_write(abi);
+        let access_file = AccessFs::from_file(abi);
         assert_eq!(access_read, !access_write & access_all);
         assert_eq!(access_read | access_write, access_all);
+        assert_eq!(access_file, access_all & ACCESS_FILE);
     }
 }
 
@@ -196,6 +205,126 @@ impl PrivateAccess for AccessFs {
     }
 }
 
+impl fmt::Display for AccessFs {
+    #[allow(clippy::cognitive_complexity)]
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        // Collect all flag-names in the same order used
+        // when building `access` for the Syd sandbox.
+        let mut parts = Vec::new();
+
+        if self.contains(AccessFs::ReadFile) {
+            parts.push("read");
+        }
+        if self.contains(AccessFs::WriteFile) {
+            parts.push("write");
+        }
+        if self.contains(AccessFs::Execute) {
+            parts.push("exec");
+        }
+        if self.contains(AccessFs::IoctlDev) {
+            parts.push("ioctl");
+        }
+        if self.contains(AccessFs::MakeReg) {
+            parts.push("create");
+        }
+        if self.contains(AccessFs::RemoveFile) {
+            parts.push("delete");
+        }
+        if self.contains(AccessFs::Refer) {
+            parts.push("rename");
+        }
+        if self.contains(AccessFs::MakeSym) {
+            parts.push("symlink");
+        }
+        if self.contains(AccessFs::Truncate) {
+            parts.push("truncate");
+        }
+        if self.contains(AccessFs::ReadDir) {
+            parts.push("readdir");
+        }
+        if self.contains(AccessFs::MakeDir) {
+            parts.push("mkdir");
+        }
+        if self.contains(AccessFs::RemoveDir) {
+            parts.push("rmdir");
+        }
+        if self.contains(AccessFs::MakeChar) {
+            parts.push("mkdev");
+        }
+        if self.contains(AccessFs::MakeFifo) {
+            parts.push("mkfifo");
+        }
+        if self.contains(AccessFs::MakeSock) {
+            parts.push("bind");
+        }
+
+        // Join all set-flag names with commas.
+        write!(f, "{}", parts.join(", "))
+    }
+}
+
+impl Serialize for AccessFs {
+    #[allow(clippy::cognitive_complexity)]
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        // Collect all flag-names in the same order used
+        // when building `access` for the Syd sandbox.
+        let mut items = Vec::new();
+
+        if self.contains(Self::ReadFile) {
+            items.push("read");
+        }
+        if self.contains(Self::WriteFile) {
+            items.push("write");
+        }
+        if self.contains(Self::Execute) {
+            items.push("exec");
+        }
+        if self.contains(Self::IoctlDev) {
+            items.push("ioctl");
+        }
+        if self.contains(Self::MakeReg) {
+            items.push("create");
+        }
+        if self.contains(Self::RemoveFile) {
+            items.push("delete");
+        }
+        if self.contains(Self::Refer) {
+            items.push("rename");
+        }
+        if self.contains(Self::MakeSym) {
+            items.push("symlink");
+        }
+        if self.contains(Self::Truncate) {
+            items.push("truncate");
+        }
+        if self.contains(Self::ReadDir) {
+            items.push("readdir");
+        }
+        if self.contains(Self::MakeDir) {
+            items.push("mkdir");
+        }
+        if self.contains(Self::RemoveDir) {
+            items.push("rmdir");
+        }
+        if self.contains(Self::MakeChar) {
+            items.push("mkdev");
+        }
+        if self.contains(Self::MakeFifo) {
+            items.push("mkfifo");
+        }
+        if self.contains(Self::MakeSock) {
+            items.push("bind");
+        }
+
+        // Delegate serialization to Vec<&str>.
+        // This serializes as a JSON array of strings.
+        items.serialize(serializer)
+    }
+}
+
 // TODO: Make ACCESS_FILE a property of AccessFs.
 // TODO: Add tests for ACCESS_FILE.
 const ACCESS_FILE: AccessFs = make_bitflags!(AccessFs::{
@@ -207,13 +336,9 @@ fn is_file<F>(fd: F) -> Result<bool, Error>
 where
     F: AsFd,
 {
-    unsafe {
-        let mut stat = zeroed();
-        match libc::fstat(fd.as_fd().as_raw_fd(), &mut stat) {
-            0 => Ok((stat.st_mode & libc::S_IFMT) != libc::S_IFDIR),
-            _ => Err(Error::last_os_error()),
-        }
-    }
+    file_type(fd, None, false)
+        .map(|typ| typ != FileType::Dir)
+        .map_err(|errno| Error::from_raw_os_error(errno as i32))
 }
 
 /// Landlock rule for a file hierarchy.
@@ -321,7 +446,7 @@ fn path_beneath_try_compat_children() {
     // Do not actually perform any syscall.
     ruleset.compat.state = CompatState::Dummy;
     assert!(matches!(
-        RulesetCreated::new(ruleset, -1)
+        RulesetCreated::new(ruleset, None)
             .set_compatibility(CompatLevel::HardRequirement)
             .add_rule(PathBeneath::new(PathFd::new("/dev/null").unwrap(), access_file))
             .unwrap_err(),
@@ -335,7 +460,7 @@ fn path_beneath_try_compat_children() {
     // Do not actually perform any syscall.
     ruleset.compat.state = CompatState::Dummy;
     assert!(matches!(
-        RulesetCreated::new(ruleset, -1)
+        RulesetCreated::new(ruleset, None)
             .set_compatibility(CompatLevel::HardRequirement)
             .add_rule(PathBeneath::new(PathFd::new("/dev/null").unwrap(), access_file))
             .unwrap_err(),
@@ -519,10 +644,6 @@ impl PathFd {
                 OFlag::O_PATH | OFlag::O_CLOEXEC,
                 Mode::empty(),
             )
-            .map(|fd| {
-                // SAFETY: open returns a valid FD.
-                unsafe { OwnedFd::from_raw_fd(fd) }
-            })
             .map_err(|e| PathFdError::OpenCall {
                 source: Error::from_raw_os_error(e as i32),
                 path: path.as_ref().into(),
@@ -563,8 +684,8 @@ fn path_fd() {
 ///
 /// ```
 /// use syd::landlock::{
-///     path_beneath_rules, Access, AccessFs, Ruleset, RulesetAttr, RulesetCreatedAttr,
-///     RulesetError, RulesetStatus, ABI,
+///     path_beneath_rules, Access, AccessFs, RestrictSelfFlags, Ruleset, RulesetAttr,
+///     RulesetCreatedAttr, RulesetError, RulesetStatus, ABI,
 /// };
 ///
 /// fn restrict_thread() -> Result<(), RulesetError> {
@@ -576,7 +697,7 @@ fn path_fd() {
 ///         .add_rules(path_beneath_rules(&["/usr", "/etc", "/dev"], AccessFs::from_read(abi)))?
 ///         // Read-write access to /home and /tmp.
 ///         .add_rules(path_beneath_rules(&["/home", "/tmp"], AccessFs::from_all(abi)))?
-///         .restrict_self()?;
+///         .restrict_self(RestrictSelfFlags::empty())?;
 ///     match status.ruleset {
 ///         // The FullyEnforced case must be tested by the developer.
 ///         RulesetStatus::FullyEnforced => println!("Fully sandboxed."),
