@@ -9,20 +9,24 @@
 use std::{
     io::Write,
     os::fd::{BorrowedFd, RawFd},
+    sync::OnceLock,
+    thread::ThreadId,
     time::{SystemTime, UNIX_EPOCH},
 };
 
+use btoi::btoi;
 use data_encoding::HEXLOWER;
 use nix::{
     errno::Errno,
-    unistd::{fdatasync, write, Pid, Uid},
+    unistd::{write, Pid, Uid},
 };
 use serde_json::{Map, Value};
 
 use crate::{
     config::*,
-    fs::{lock_fd, retry_on_eintr, unlock_fd},
-    proc::*,
+    err::SydResult,
+    fs::{is_active_fd, lock_fd, unlock_fd},
+    proc::{proc_cmdline, proc_comm, proc_cwd, proc_tty},
     syslog::LogLevel,
 };
 
@@ -32,6 +36,61 @@ pub(crate) static LOG_TTY: std::sync::atomic::AtomicBool =
 // Log destination file descriptor.
 pub(crate) static LOG_FD: std::sync::atomic::AtomicI32 = std::sync::atomic::AtomicI32::new(-42);
 
+// Main thread identifier, used to decide on panic hook.
+pub(crate) static LOG_MAIN_TID: OnceLock<ThreadId> = OnceLock::new();
+
+// Set this in main thread after namespace forks and other dances.
+pub(crate) fn log_init_main() -> SydResult<()> {
+    LOG_MAIN_TID
+        .set(std::thread::current().id())
+        .map_err(|_| Errno::EBUSY.into())
+}
+
+// Sets the global panic hook for Syd threads.
+pub(crate) fn log_set_panic_hook() {
+    // Set a logging panic hook. The default panic
+    // hook calls system calls not permitted by emulators
+    // such as getcwd(2), stat(2) etc.
+    #[allow(clippy::cognitive_complexity)]
+    std::panic::set_hook(Box::new(|info| {
+        let this = std::thread::current();
+        let name = this.name().unwrap_or("?");
+
+        let err = match info.payload().downcast_ref::<&'static str>() {
+            Some(s) => *s,
+            None => match info.payload().downcast_ref::<String>() {
+                Some(s) => &**s,
+                None => "?",
+            },
+        };
+
+        let file = info.location().map(|l| l.file());
+        let line = info.location().map(|l| l.line());
+
+        if log_is_main() {
+            // Main thread panicing isn't recoverable unlike others.
+            // Ensure clean exit right away.
+            crate::alert!("ctx": "panic",
+                "op": "panic_syd_main_thread",
+                "msg": err, "file": file, "line": line);
+            std::process::exit(101);
+        } else {
+            crate::crit!("ctx": "panic",
+                "op": format!("panic_{name}_thread"),
+                "msg": err, "file": file, "line": line);
+        }
+    }));
+}
+
+// Use this to check for main thread in panic handler.
+#[inline]
+fn log_is_main() -> bool {
+    LOG_MAIN_TID
+        .get()
+        .map(|&tid| tid == std::thread::current().id())
+        .unwrap_or(false)
+}
+
 /// emerg! logging macro
 #[macro_export]
 macro_rules! emerg {
@@ -226,13 +285,7 @@ impl Write for LockedWriter<'_> {
     }
 
     fn flush(&mut self) -> std::io::Result<()> {
-        // SAFETY:
-        // 1. Retry on EINTR.
-        // 2. Ignore EINVAL which indicates special file.
-        match retry_on_eintr(|| fdatasync(self.fd)) {
-            Ok(_) | Err(Errno::EINVAL) => Ok(()),
-            Err(e) => Err(std::io::Error::from_raw_os_error(e as i32)),
-        }
+        Ok(())
     }
 }
 
@@ -252,15 +305,23 @@ pub fn log_init(default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resul
         default_level
     };
 
-    // Determine if syslog host forwarding is on or off
-    let host_syslog = std::env::var_os(ENV_NO_SYSLOG).is_none();
-
     // Determine the main FD for logging:
-    // By default, we use stderr. If ENV_LOG_FD is set, we parse it.
+    // By default, we use stderr.
+    // If ENV_LOG_FD is set, we parse it:
+    // 1. negative fd is ok as a shorthand to disable logging.
+    // 2. positive fd must be a valid fd or we bail with EBADF.
     let fd = match std::env::var_os(ENV_LOG_FD) {
         None => default_log_fd,
         Some(val) => {
-            Some(btoi::btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?)
+            let fd = btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?;
+            if fd >= 0 {
+                // SAFETY: fd only used after validation.
+                let fd = unsafe { BorrowedFd::borrow_raw(fd) };
+                if !is_active_fd(fd) {
+                    return Err(Errno::EBADF);
+                }
+            }
+            Some(fd)
         }
     };
     if let Some(fd) = fd {
@@ -299,7 +360,7 @@ pub fn log_init(default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resul
             .or(Err(Errno::EINVAL))?;
         use_stack = false;
     }
-    init_global_syslog(logbuflen, fd, level, host_syslog, use_stack)?;
+    init_global_syslog(logbuflen, fd, level, use_stack)?;
 
     // Finally let's make some noise!
     info!("ctx": "init", "op": "sing", "chapter": 24,
@@ -325,14 +386,24 @@ pub fn log_init_simple(default_level: LogLevel) -> Result<(), Errno> {
         default_level
     };
 
-    // We'll turn OFF host syslog, always.
-    let host_syslog = false;
-
     // Determine the main FD for logging:
-    // By default, we use stderr. If ENV_LOG_FD is set, we parse it.
+    // By default, we use stderr.
+    // If ENV_LOG_FD is set, we parse it:
+    // 1. negative fd is ok as a shorthand to disable logging.
+    // 2. positive fd must be a valid fd or we bail with EBADF.
     let fd = match std::env::var_os(ENV_LOG_FD) {
         None => libc::STDERR_FILENO,
-        Some(val) => btoi::btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?,
+        Some(val) => {
+            let fd = btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?;
+            if fd >= 0 {
+                // SAFETY: fd only used after validation.
+                let fd = unsafe { BorrowedFd::borrow_raw(fd) };
+                if !is_active_fd(fd) {
+                    return Err(Errno::EBADF);
+                }
+            }
+            fd
+        }
     };
     LOG_FD.store(fd, std::sync::atomic::Ordering::Relaxed);
 
@@ -352,7 +423,7 @@ pub fn log_init_simple(default_level: LogLevel) -> Result<(), Errno> {
     LOG_TTY.store(tty, std::sync::atomic::Ordering::Relaxed);
 
     // Create a global syslog with ring.
-    init_global_syslog(0, Some(fd), level, host_syslog, true)?;
+    init_global_syslog(0, Some(fd), level, true)?;
 
     if let Some(sys) = global_syslog() {
         // Now lock it immediately,
@@ -367,6 +438,12 @@ pub fn log_init_simple(default_level: LogLevel) -> Result<(), Errno> {
 #[cfg(feature = "log")]
 #[allow(clippy::cognitive_complexity)]
 pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String, Value>) {
+    let sys = if let Some(sys) = crate::syslog::global_syslog() {
+        sys
+    } else {
+        return; // Logging is disabled.
+    };
+
     // For "higher" severities, we add more contextual fields.
     let add_context = level.as_u8() <= crate::syslog::LogLevel::Notice.as_u8();
     let tty = LOG_TTY.load(std::sync::atomic::Ordering::Relaxed);
@@ -393,6 +470,13 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
             } else {
                 msg.insert("cwd".to_string(), Value::Null);
             }
+            if add_context {
+                if let Ok(tty) = proc_tty(pid) {
+                    msg.insert("tty".to_string(), Value::String(tty.to_string()));
+                } else {
+                    msg.insert("tty".to_string(), Value::Null);
+                }
+            }
         }
         msg.insert("pid".to_string(), Value::Number(pid.as_raw().into()));
     }
@@ -441,9 +525,7 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
         None
     };
 
-    if let Some(sys) = crate::syslog::global_syslog() {
-        sys.write_log(level, &msg_data, msg_pretty.as_deref());
-    }
+    sys.write_log(level, &msg_data, msg_pretty.as_deref());
 }
 
 /// Initializes Syslog global object, reading environment variables.
@@ -454,11 +536,22 @@ pub fn log_init(_default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resu
     use std::os::unix::ffi::OsStrExt;
 
     // Determine the main FD for logging:
-    // By default, we use stderr. If ENV_LOG_FD is set, we parse it.
+    // By default, we use stderr.
+    // If ENV_LOG_FD is set, we parse it:
+    // 1. negative fd is ok as a shorthand to disable logging.
+    // 2. positive fd must be a valid fd or we bail with EBADF.
     let fd = match std::env::var_os(ENV_LOG_FD) {
         None => default_log_fd,
         Some(val) => {
-            Some(btoi::btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?)
+            let fd = btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?;
+            if fd >= 0 {
+                // SAFETY: fd only used after validation.
+                let fd = unsafe { BorrowedFd::borrow_raw(fd) };
+                if !is_active_fd(fd) {
+                    return Err(Errno::EBADF);
+                }
+            }
+            Some(fd)
         }
     };
     if let Some(fd) = fd {
@@ -482,11 +575,22 @@ pub fn log_init(_default_level: LogLevel, default_log_fd: Option<RawFd>) -> Resu
     LOG_TTY.store(tty, std::sync::atomic::Ordering::Relaxed);
 
     // Determine the main FD for logging:
-    // By default, we use stderr. If ENV_LOG_FD is set, we parse it.
+    // By default, we use stderr.
+    // If ENV_LOG_FD is set, we parse it:
+    // 1. negative fd is ok as a shorthand to disable logging.
+    // 2. positive fd must be a valid fd or we bail with EBADF.
     let fd = match std::env::var_os(ENV_LOG_FD) {
         None => default_log_fd,
         Some(val) => {
-            Some(btoi::btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?)
+            let fd = btoi::<RawFd>(val.as_os_str().as_bytes()).map_err(|_| Errno::EBADF)?;
+            if fd >= 0 {
+                // SAFETY: fd only used after validation.
+                let fd = unsafe { BorrowedFd::borrow_raw(fd) };
+                if !is_active_fd(fd) {
+                    return Err(Errno::EBADF);
+                }
+            }
+            Some(fd)
         }
     };
     if let Some(fd) = fd {
@@ -511,6 +615,15 @@ pub fn log_init_simple(default_level: LogLevel) -> Result<(), Errno> {
 #[cfg(not(feature = "log"))]
 #[allow(clippy::cognitive_complexity)]
 pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String, Value>) {
+    // Setting LOG-FD to negative is OK to disable logging.
+    let fd = LOG_FD.load(std::sync::atomic::Ordering::Relaxed);
+    let fd = if fd < 0 {
+        return; // Logging is disabled.
+    } else {
+        // SAFETY: we trust log fd is valid.
+        unsafe { BorrowedFd::borrow_raw(fd) }
+    };
+
     // For "higher" severities, we add more contextual fields.
     let add_context = level.as_u8() <= crate::syslog::LogLevel::Notice.as_u8();
     let tty = LOG_TTY.load(std::sync::atomic::Ordering::Relaxed);
@@ -537,6 +650,13 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
             } else {
                 msg.insert("cwd".to_string(), Value::Null);
             }
+            if add_context {
+                if let Ok(tty) = proc_tty(pid) {
+                    msg.insert("tty".to_string(), Value::String(tty.to_string()));
+                } else {
+                    msg.insert("tty".to_string(), Value::Null);
+                }
+            }
         }
         msg.insert("pid".to_string(), Value::Number(pid.as_raw().into()));
     }
@@ -587,18 +707,13 @@ pub fn log(level: crate::syslog::LogLevel, timestamp: u64, mut msg: Map<String,
     };
 
     // Finally, log to fd or standard error.
-    let fd = LOG_FD.load(std::sync::atomic::Ordering::Relaxed);
-    if fd >= 0 {
-        // SAFETY: we trust log fd is valid.
-        let fd = unsafe { BorrowedFd::borrow_raw(fd) };
-        if let Ok(mut writer) = LockedWriter::new(fd).map(std::io::BufWriter::new) {
-            if let Some(ref msg) = msg_info {
-                let _ = writer.write_all(msg.as_bytes());
-            } else {
-                let _ = writer.write_all(msg_data.as_bytes());
-            }
-            let _ = writer.write_all(b"\n");
+    if let Ok(mut writer) = LockedWriter::new(fd).map(std::io::BufWriter::new) {
+        if let Some(ref msg) = msg_info {
+            let _ = writer.write_all(msg.as_bytes());
+        } else {
+            let _ = writer.write_all(msg_data.as_bytes());
         }
+        let _ = writer.write_all(b"\n");
     }
 }
 
