@@ -51,7 +51,6 @@
 #![deny(clippy::undocumented_unsafe_blocks)]
 
 use std::{
-    collections::{HashMap, HashSet},
     env,
     ffi::{CString, OsStr, OsString},
     fs::{File, OpenOptions},
@@ -87,20 +86,21 @@ use syd::{
     caps,
     compat::{waitid, WaitStatus},
     config::*,
+    confine::{nsflags_name, ExportMode},
     err::err2no,
     error,
     fs::{closeexcept, pidfd_open, retry_on_eintr, set_cloexec},
-    hash::SydRandomState,
+    hash::{SydHashMap, SydHashSet},
     hook::Supervisor,
     info,
     landlock_policy::LandlockPolicy,
     log::log_init,
-    loopback_set_up, nsflags_name,
+    loopback_set_up,
     path::XPathBuf,
     sandbox::Sandbox,
     syslog::LogLevel,
     unshare::{GidMap, UidMap},
-    warn, ExportMode,
+    warn,
 };
 
 // Set global allocator to tcmalloc if profiling is enabled.
@@ -200,7 +200,7 @@ syd::main! {
     let mut sandbox: Sandbox = Sandbox::default();
     let mut cmd_arg0: Option<OsString> = None;
     let mut cmd_argv: Vec<OsString> = vec![];
-    let mut cmd_envp: HashSet<OsString, SydRandomState> = HashSet::default();
+    let mut cmd_envp: SydHashSet<OsString> = SydHashSet::default();
 
     // SYD_PROXY_{HOST,PORT,UNIX} -> proxy/ext/{host,port,unix}
     #[allow(clippy::disallowed_methods)]
@@ -428,10 +428,10 @@ syd::main! {
 
     // SAFETY: We cannot support NEWPID without NEWNS.
     // ie, pid namespace must have its own private /proc.
-    if sandbox.unshare_pid() {
+    if sandbox.flags.unshare_pid() {
         sandbox.set_unshare_mount(true);
     }
-    if sandbox.unshare_mount() {
+    if sandbox.flags.unshare_mount() {
         sandbox.set_unshare_pid(true);
     }
 
@@ -462,7 +462,7 @@ syd::main! {
         // 2. LD_LIBRARY_PATH
         // 3. SYD_PTY_DEBUG
         // 4. SYD_PTY_RULES
-        let safe_env: HashMap<_, _, SydRandomState> = env::vars_os()
+        let safe_env: SydHashMap<_, _> = env::vars_os()
             .filter(|(key, _)| {
                 matches!(
                     key.as_bytes(),
@@ -556,7 +556,7 @@ syd::main! {
         // 2. LD_LIBRARY_PATH
         // 3. SYD_TOR_DEBUG
         // 4. SYD_TOR_RULES
-        let safe_env: HashMap<_, _, SydRandomState> = env::vars_os()
+        let safe_env: SydHashMap<_, _> = env::vars_os()
             .filter(|(key, _)| {
                 matches!(
                     key.as_bytes(),
@@ -622,7 +622,7 @@ syd::main! {
     // include the Syd process into the pid namespace as well
     // such that the sandbox process and syd have the identical
     // view of /proc.
-    let namespaces = sandbox.namespaces();
+    let namespaces = sandbox.flags.namespaces();
     if namespaces == 0 {
         // No namespace arguments passed, run normally.
         return match Supervisor::run(
@@ -648,12 +648,12 @@ syd::main! {
             "msg": format!("setting up Syd container with {n} namespace{s}"));
     }
 
-    let id_buf = if sandbox.unshare_user() {
+    let id_buf = if sandbox.flags.unshare_user() {
         // create the UID and GID mappings.
         let uid = getuid().as_raw();
         let gid = getgid().as_raw();
 
-        let map_root = sandbox.map_root();
+        let map_root = sandbox.flags.map_root();
 
         let uid_buf = {
             let uid_maps = vec![
@@ -717,15 +717,16 @@ syd::main! {
         // Set inheritable mask and ambient caps to retain caps after execve(2).
         caps::securebits::set_keepcaps(true)?;
         let permitted_caps = caps::read(None, caps::CapSet::Permitted)?;
-        caps::set(None, caps::CapSet::Inheritable, &permitted_caps)?;
+        caps::set(None, caps::CapSet::Inheritable, permitted_caps)?;
         // Set the same capabilities as ambient, if necessary.
-        for cap in permitted_caps {
+        for flag in permitted_caps {
+            let cap = flag.try_into()?;
             caps::raise(None, caps::CapSet::Ambient, cap)?;
         }
     }
 
     // Bring up loopback device for net namespace.
-    if sandbox.unshare_net() {
+    if sandbox.flags.unshare_net() {
         // Set up the loopback interface.
         // Warn on errors and continue.
         match loopback_set_up() {
@@ -814,7 +815,7 @@ syd::main! {
 
     // Set host and domain name for uts namespace.
     // Ignore errors.
-    if sandbox.unshare_uts() {
+    if sandbox.flags.unshare_uts() {
         // SAFETY: Domain name must not contain a nul-byte.
         #[allow(clippy::disallowed_methods)]
         let domainname = CString::new(sandbox.domainname.clone()).unwrap();
@@ -824,7 +825,7 @@ syd::main! {
         let _ = sethostname(&sandbox.hostname);
     }
 
-    if sandbox.unshare_time() {
+    if sandbox.flags.unshare_time() {
         #[allow(clippy::disallowed_methods)]
         if let Some(offset) = sandbox.time {
             let mut file = File::create("/proc/self/timens_offsets")?;
@@ -844,7 +845,7 @@ syd::main! {
 
     // Disable Speculative Store Bypass mitigations
     // for trace/allow_unsafe_spec_exec:1
-    let ssb = sandbox.allow_unsafe_spec_exec();
+    let ssb = sandbox.flags.allow_unsafe_spec_exec();
 
     // SAFETY: fork is our safest option here.
     let child = match unsafe { fork() }? {
@@ -958,15 +959,13 @@ syd::main! {
         }
     }
 
-    // Allow write to standard error.
-    for name in ["write", "writev"] {
-        if let Ok(syscall) = ScmpSyscall::from_name(name) {
-            ctx.add_rule_conditional(
-                ScmpAction::Allow,
-                syscall,
-                &[scmp_cmp!($arg0 == libc::STDERR_FILENO as u64)],
-            )?;
-        }
+    // Allow write(2) to standard error.
+    if let Ok(syscall) = ScmpSyscall::from_name("write") {
+        ctx.add_rule_conditional(
+            ScmpAction::Allow,
+            syscall,
+            &[scmp_cmp!($arg0 == libc::STDERR_FILENO as u64)],
+        )?;
     }
 
     // Prevent executable memory.
