@@ -14,11 +14,6 @@
 // to synchronize concurrent producers (writers) and consumers
 // (readers).
 //
-// Additionally, if the `host_syslog` parameter is set to true at
-// startup, we will send every message at level Warn (4) and above to
-// the host's system syslog using `libc::syslog` function (which we do
-// NOT re- implement; we just call it directly, unformatted).
-//
 // SECURITY & CONCURRENCY DISCLAIMER:
 //   - The ring buffer is shared among multiple producer threads (which
 //   take a write lock) and multiple consumer threads (which typically
@@ -27,8 +22,7 @@
 //   locks. We minimize lock hold times.
 //   - Once locked (via `lock()`), the ring buffer is freed and any
 //   subsequent ring-based operations return EPERM. Writes to the main
-//   fd + host_syslog continue. Writes to the console fd do NOT continue
-//   after locked.
+//   fd continue. Writes to the console fd do NOT continue after locked.
 //
 // If the "log" feature is not enabled, we provide stubs that return ENOSYS.
 
@@ -234,7 +228,6 @@ mod syslog_enabled {
 
     use nix::{
         errno::Errno,
-        syslog::{openlog, syslog, Facility, LogFlags, Priority, Severity},
         time::{clock_gettime, ClockId},
     };
     use ringbuf::{
@@ -246,39 +239,6 @@ mod syslog_enabled {
 
     use crate::{config::SYSLOG_STACK_SIZE, log::LockedWriter, syslog::*};
 
-    /// Opens a socket connection to the system's syslog.
-    ///
-    /// Note, if this connection ever gets broken the
-    /// subsequent `host_syslog` call is going to cause
-    /// instant termination of the Syd process and all
-    /// processes running under it. This may be an issue
-    /// long-term, therefore:
-    ///
-    /// TODO: Implement our own system.syslog service
-    ///       a la Capsicum's libcasper.
-    fn open_syslog() -> Result<(), Errno> {
-        // LOG_PID: Log process ID.
-        // LOG_NDELAY: Open connection to syslogd(8) immediately,
-        //             because the default behaviour of openlog(3)
-        //             is to delay opening the connection until
-        //             the first `host_syslog` call in which case
-        //             our socket(2) seccomp filters will gladly
-        //             terminate us.
-        openlog(
-            Some(c"syd"),
-            LogFlags::LOG_PID | LogFlags::LOG_NDELAY,
-            Facility::LOG_USER,
-        )
-    }
-
-    /// Logs a message to the system's syslog.
-    fn host_syslog(message: &str) -> Result<(), Errno> {
-        syslog(
-            Priority::new(Severity::LOG_WARNING, Facility::LOG_USER),
-            message,
-        )
-    }
-
     // Store ring data in an enum to handle "heap" vs "static" capacity.
     // Store Arc<SharedRb<...>> plus Caching wrappers to fix all trait bounds.
     enum RbMode {
@@ -304,8 +264,6 @@ mod syslog_enabled {
     ///  - Many producers each do ring_lock.write() to push logs.
     ///  - Many consumers do ring_lock.read() to read logs.
     ///  - read_clear or clear requires write lock.
-    ///  - If `host_syslog` is true => every message that is level >= Warn
-    ///    is also sent to the host's syslog with NO extra formatting.
     ///  - If locked => ring is freed => ring ops => EPERM.
     ///    We still write to fd + host syslog after locking.
     pub struct Syslog {
@@ -317,9 +275,6 @@ mod syslog_enabled {
 
         // Log level.
         level: AtomicU8,
-
-        // Send messages to host syslog.
-        host_syslog: AtomicBool,
     }
 
     // SAFETY: The ringbuf uses interior mutability, but SharedRb + Arc
@@ -331,13 +286,10 @@ mod syslog_enabled {
         /// - If `use_stack == true`, uses a SharedRb<Array<u8, SYSLOG_STACK_SIZE>>.
         /// - Otherwise, uses a SharedRb<Heap<u8>> of capacity `capacity`.
         /// - `fd` is an optional raw fd for raw user messages (always used).
-        /// - If `host_syslog == true`, we call `syslog(message)` for
-        ///   messages that are Warn or higher (Warn=4..Emergent=0).
         pub fn new(
             capacity: usize,
             fd: Option<std::os::fd::RawFd>,
             level: LogLevel,
-            host_syslog: bool,
             use_stack: bool,
         ) -> Self {
             let ring_data = if use_stack {
@@ -369,7 +321,6 @@ mod syslog_enabled {
                     unsafe { BorrowedFd::borrow_raw(fd) }
                 }),
                 level: AtomicU8::new(level as u8),
-                host_syslog: AtomicBool::new(host_syslog),
             }
         }
 
@@ -377,7 +328,6 @@ mod syslog_enabled {
         /// - Always writes the raw message to the `fd` (if present).
         /// - If not locked, also writes the formatted ring message to the ring,
         ///   under a short write lock.
-        /// - If `host_syslog` is true and `level >= Warn`, call `host_syslog(msg)`.
         ///
         /// If locked => no ring operations, but still writes to fd + host syslog.
         pub fn write_log(&self, level: LogLevel, msg: &str, msg_pretty: Option<&str>) {
@@ -386,25 +336,6 @@ mod syslog_enabled {
                 return;
             }
 
-            // Always write to host syslog if log level is Warn or higher.
-            // Do not use formatting for syslog messages.
-            if self.host_syslog() && level.as_u8() <= LogLevel::Warn.as_u8() {
-                // Call host syslog with no formatting:
-                //
-                // SAFETY:
-                // 1. We use openlog(3) at startup with LOG_NDELAY,
-                //    and an attempt to reconnect to the syslogd(3) via
-                //    syslog(3) will cause instant Syd termination together
-                //    with all the processes running under it.
-                // 2. Terminating the process here with a panic is arguably
-                //    the best we can do given we do not want to continue
-                //    without the host visibility and next socket(2) is doomed
-                //    to kill us anyway.
-                //    TODO: system.syslog service a la Capsicum's libcasper.
-                #[allow(clippy::disallowed_methods)]
-                host_syslog(msg).expect("syslogd");
-            }
-
             // Always attempt to write message to fd, append a new line to the message.
             // Use formatting as necessary.
             // We take a OFD write lock here.
@@ -599,16 +530,6 @@ mod syslog_enabled {
             self.level.store(lv, Ordering::SeqCst);
         }
 
-        /// Returns the current host syslog setting.
-        pub fn host_syslog(&self) -> bool {
-            self.host_syslog.load(Ordering::SeqCst)
-        }
-
-        /// Sets the host syslog setting.
-        pub fn set_host_syslog(&self, state: bool) {
-            self.host_syslog.store(state, Ordering::SeqCst);
-        }
-
         /// Returns true if syslog is locked.
         pub fn is_locked(&self) -> bool {
             self.locked.load(Ordering::SeqCst)
@@ -822,7 +743,6 @@ mod syslog_enabled {
         capacity: usize,
         fd: Option<std::os::fd::RawFd>,
         level: LogLevel,
-        host_syslog: bool,
         use_stack: bool,
     ) -> Result<(), Errno> {
         if !use_stack && capacity == 0 {
@@ -830,15 +750,8 @@ mod syslog_enabled {
             return Err(Errno::EINVAL);
         }
 
-        if host_syslog {
-            // SAFETY: Open syslogd(8) connection immediately with LOG_NDELAY,
-            // to ensure we can continue sending syslog(3) messages
-            // under seccomp(2) confinement with syscall cookies et al.
-            open_syslog()?;
-        }
-
         SYSLOG_INSTANCE
-            .set(Syslog::new(capacity, fd, level, host_syslog, use_stack))
+            .set(Syslog::new(capacity, fd, level, use_stack))
             .or(Err(Errno::EAGAIN))
     }
 
@@ -867,10 +780,16 @@ pub fn current_loglevel() -> u8 {
 pub fn current_loglevel() -> u8 {
     use std::os::unix::ffi::OsStrExt;
 
-    std::env::var_os(crate::config::ENV_LOG)
-        .map(|val| parse_loglevel(val.as_os_str().as_bytes(), LogLevel::Warn))
-        .unwrap_or(LogLevel::Warn)
-        .as_u8()
+    use once_cell::sync::Lazy;
+
+    static LOGLEVEL: Lazy<u8> = Lazy::new(|| {
+        std::env::var_os(crate::config::ENV_LOG)
+            .map(|val| parse_loglevel(val.as_os_str().as_bytes(), LogLevel::Warn))
+            .unwrap_or(LogLevel::Warn)
+            .as_u8()
+    });
+
+    *LOGLEVEL
 }
 
 /// Returns whether given log level is enabled.
