@@ -6,13 +6,10 @@
 //
 // SPDX-License-Identifier: GPL-3.0
 
-#![allow(clippy::disallowed_types)]
-
 use std::{
     env,
     fs::remove_dir_all,
     ops::Range,
-    path::{Path, PathBuf},
     process::{Command, ExitCode},
 };
 
@@ -20,9 +17,14 @@ use nix::{
     errno::Errno,
     fcntl::AT_FDCWD,
     sys::stat::{fchmodat, umask, FchmodatFlags, Mode},
-    unistd::{chdir, mkdir, mkdtemp},
+    unistd::{chdir, getcwd, mkdir, mkdtemp},
+};
+use syd::{
+    err::SydResult,
+    path::{XPath, XPathBuf},
+    syslog::LogLevel,
+    wildmatch::inamematch,
 };
-use syd::{err::SydResult, syslog::LogLevel, wildmatch::inamematch};
 
 use crate::util::shuffle_vec;
 
@@ -32,15 +34,15 @@ use test::*;
 
 #[derive(Debug)]
 struct TempDir {
-    path: PathBuf,
+    path: XPathBuf,
     keep: bool,
 }
 
 impl TempDir {
-    fn new<P: AsRef<Path>>(path: P) -> Self {
+    fn new(path: &XPath) -> Self {
         TempDir {
-            path: path.as_ref().to_path_buf(),
-            keep: false,
+            path: path.to_owned(),
+            keep: std::env::var_os("SYD_TEST_KEEP").is_some(),
         }
     }
 }
@@ -161,10 +163,10 @@ fn main() -> SydResult<ExitCode> {
             "{}/syd_test_XXXXXX",
             env::var("SYD_TEST_TMPDIR").unwrap_or(".".to_string())
         );
-        match mkdtemp(Path::new(&tmp)) {
+        match mkdtemp(XPath::new(&tmp)) {
             Ok(path) => {
                 // HOME must be canonicalized!
-                let path = path.canonicalize()?;
+                let path = path.canonicalize().map(XPathBuf::from)?;
 
                 match chdir(&path) {
                     Ok(_) => {
@@ -172,7 +174,7 @@ fn main() -> SydResult<ExitCode> {
                         println!("# Use SYD_TEST_TMPDIR to override.");
                         //Setting HOME here breaks podman!
                         //env::set_var("HOME", &path);
-                        Some(TempDir::new(path))
+                        Some(TempDir::new(&path))
                     }
                     Err(error) => {
                         println!("# chdir failed: {error}.");
@@ -290,7 +292,7 @@ fn main() -> SydResult<ExitCode> {
     shuffle_vec(&mut test_indices);
 
     // Print TAP plan.
-    if syd::syd_enabled() {
+    if syd::confine::syd_enabled() {
         println!("1..0 # SKIP syd is enabled!");
         return Ok(ExitCode::from(0));
     }
@@ -369,7 +371,14 @@ fn main() -> SydResult<ExitCode> {
                 } else {
                     println!("ok {idx} - {name}");
                 }
-                let _ = remove_dir_all(name);
+                if tmpdir.as_ref().map(|t| t.keep).unwrap_or(false) {
+                    let cwd = getcwd()
+                        .map(XPathBuf::from)
+                        .unwrap_or_else(|_| XPathBuf::from("?"));
+                    eprintln!("# Keeping test directory \"{cwd}\"");
+                } else {
+                    let _ = remove_dir_all(name);
+                }
             }
             Err(error) => {
                 println!("not ok {idx} - {name} - FAIL: {error}");
