@@ -19,7 +19,7 @@ use nix::{
 };
 
 use crate::{
-    caps,
+    caps::{securebits::set_keepcaps, Capability},
     config::*,
     confine::{confine_scmp_ioctl, confine_scmp_pwritev2, safe_drop_cap},
     fs::retry_on_eintr,
@@ -323,11 +323,33 @@ pub extern "C" fn child_after_clone(arg: *mut libc::c_void) -> libc::c_int {
         drop(pipe_rw);
     }
 
+    // SAFETY: Drop the following capabilities unconditionally.
+    // 1. CAP_CHOWN: for privileged chown(2)
+    // 2. CAP_MKNOD: for privileged mknod(2)
+    // 3. CAP_NET_BIND_SERVICE: for privileged bind(2)
+    // 4. CAP_NET_RAW: for privileged socket(2)
+    // These system calls happen in syd-emulator threads
+    // even if the respective unsafe options are set,
+    // therefore dropping the caps here ensures this.
+    const CAP_DROP: &[Capability] = &[
+        Capability::CAP_CHOWN,
+        Capability::CAP_MKNOD,
+        Capability::CAP_NET_BIND_SERVICE,
+        Capability::CAP_NET_RAW,
+    ];
+    for cap in CAP_DROP {
+        if safe_drop_cap(*cap).is_err() {
+            fail_safe!(child, Err::CapSet);
+        }
+    }
+
     // Drop CAP_SYS_PTRACE late as Syd may need it.
-    if !child.cfg.keep && safe_drop_cap(caps::Capability::CAP_SYS_PTRACE).is_err() {
+    if !child.cfg.keep && safe_drop_cap(Capability::CAP_SYS_PTRACE).is_err() {
         fail_safe!(child, Err::CapSet);
     }
-    if caps::securebits::set_keepcaps(true).is_err() {
+
+    // Ensure we keep the specified capabilities.
+    if set_keepcaps(true).is_err() {
         fail_safe!(child, Err::CapSet);
     }
 
