@@ -1495,7 +1495,6 @@ syd::main! {
     let mut opts = Opts::parse();
 
     // Initialize sandbox logging and Youki tracing.
-    env::set_var(syd::config::ENV_NO_SYSLOG, "NoThanks");
     let (level, trace_level) = if opts.global.debug {
         (LogLevel::Debug, tracing::Level::DEBUG)
     } else {
@@ -2354,7 +2353,7 @@ fn get_cgroup_path(cgroups_path: &Option<PathBuf>, container_id: &str) -> PathBu
 
 // Function to query and return capabilities
 fn query_caps() -> SydResult<Vec<String>> {
-    Ok(syd::caps::all()
+    Ok(syd::caps::Capabilities::all()
         .iter()
         .map(|cap| format!("{cap:?}"))
         .collect())
