@@ -7,7 +7,7 @@
 // SPDX-License-Identifier: GPL-3.0
 
 use std::{
-    os::unix::{ffi::OsStrExt, process::CommandExt},
+    os::unix::ffi::OsStrExt,
     process::{Command, ExitCode},
 };
 
@@ -17,18 +17,15 @@ use nix::{
     sched::{setns, CloneFlags},
     unistd::Pid,
 };
-#[allow(clippy::disallowed_types)]
-use procfs::process::Process;
 use syd::{
     config::SYD_SH,
+    confine::{run_cmd, CLONE_NEWTIME},
     err::SydResult,
     fs::pidfd_open,
     path::{XPath, XPathBuf},
+    proc::proc_namespaces,
 };
 
-// Not defined by nix yet.
-const CLONE_NEWTIME: CloneFlags = CloneFlags::from_bits_retain(syd::CLONE_NEWTIME);
-
 syd::main! {
     use lexopt::prelude::*;
 
@@ -108,42 +105,14 @@ syd::main! {
         opt_cmd = vec![SYD_SH.into()];
     }
     let cmd = XPathBuf::from(opt_cmd.remove(0));
+
     if opt_log {
         eprintln!("syd-run: exec command `{cmd}'...",);
     }
+
     let mut cmd = Command::new(cmd);
     let cmd = cmd.args(opt_cmd);
-
-    if namespaces.intersects(CloneFlags::CLONE_NEWPID | CLONE_NEWTIME) {
-        // Entering into pid and time namespaces require forking.
-        let mut cmd = match cmd.spawn() {
-            Ok(cmd) => cmd,
-            Err(error) => {
-                eprintln!("syd-run: spawn: {error}");
-                return Ok(ExitCode::FAILURE);
-            }
-        };
-
-        Ok(match cmd.wait() {
-            Ok(status) => {
-                if let Some(code) = status.code() {
-                    ExitCode::from(code as u8)
-                } else {
-                    ExitCode::FAILURE
-                }
-            }
-
-            Err(error) => {
-                eprintln!("syd-run: wait: {error}");
-                ExitCode::FAILURE
-            }
-        })
-    } else {
-        // Replace current binary with the new command.
-        Ok(ExitCode::from(
-            127 + cmd.exec().raw_os_error().unwrap_or(0) as u8,
-        ))
-    }
+    Ok(ExitCode::from(run_cmd(cmd)))
 }
 
 fn help() {
@@ -156,13 +125,11 @@ fn nsenter(pid: pid_t, namespaces: CloneFlags) -> Result<(), Errno> {
 }
 
 fn nsget(pid: pid_t, log: bool) -> SydResult<CloneFlags> {
-    #[allow(clippy::disallowed_types)]
-    let current_proc = Process::myself()?;
-    let current_namespaces = current_proc.namespaces()?;
+    let current_pid = Pid::this();
+    let current_namespaces = proc_namespaces(current_pid)?;
 
-    #[allow(clippy::disallowed_types)]
-    let target_proc = Process::new(pid)?;
-    let target_namespaces = target_proc.namespaces()?.0;
+    let target_pid = Pid::from_raw(pid);
+    let target_namespaces = proc_namespaces(target_pid)?.0;
 
     let mut flags = CloneFlags::empty();
 
