@@ -7,9 +7,8 @@
 // SPDX-License-Identifier: GPL-3.0
 
 use std::{
-    collections::HashMap,
     env,
-    os::fd::{AsFd, AsRawFd, FromRawFd, OwnedFd, RawFd},
+    os::fd::{AsFd, FromRawFd, OwnedFd, RawFd},
     sync::{
         atomic::{AtomicBool, Ordering},
         Arc, RwLock,
@@ -21,23 +20,22 @@ use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext, ScmpSyscall};
 use nix::{
     errno::Errno,
     fcntl::{splice, tee, OFlag, SpliceFFlags},
-    unistd::{lseek64, pipe2, write, Whence},
+    unistd::{lseek64, pipe2, write, Gid, Uid, Whence},
 };
 
 use crate::{
-    compat::ftruncate64,
     config::*,
+    confine::{confine_scmp_write, scmp_add_setid_rules, ExportMode},
     err::{err2no, SydJoinHandle, SydResult},
     error,
-    fs::{lock_fd, retry_on_eintr, retry_on_intr, seal_memfd, FileInfo},
+    fs::{ftruncate64, lock_fd, retry_on_eintr, retry_on_intr, seal_memfd, FileInfo},
     hash::{
         aes_ctr_enc, aes_ctr_init, hmac_sha256_feed, hmac_sha256_fini, hmac_sha256_init,
-        SydRandomState, BLOCK_SIZE, HMAC_TAG_SIZE, IV, IV_SIZE,
+        SydHashMap, BLOCK_SIZE, HMAC_TAG_SIZE, IV, IV_SIZE,
     },
     info,
     path::{XPath, XPathBuf},
     sandbox::Flags,
-    ExportMode,
 };
 
 #[derive(Debug, Copy, Clone, Eq, PartialEq, Ord, PartialOrd)]
@@ -61,7 +59,6 @@ impl From<OFlag> for AesMod {
     }
 }
 
-#[derive(Debug)]
 pub(crate) struct AesVal {
     pub(crate) crypt_fd: RawFd,
     pub(crate) enc_fd: Option<OwnedFd>,
@@ -71,7 +68,7 @@ pub(crate) struct AesVal {
     pub(crate) spawned: bool,
 }
 
-pub(crate) type AesMap = Arc<RwLock<HashMap<XPathBuf, AesVal, SydRandomState>>>;
+pub(crate) type AesMap = Arc<RwLock<SydHashMap<XPathBuf, AesVal>>>;
 
 #[derive(Clone)]
 pub(crate) struct AesWorker {
@@ -80,6 +77,8 @@ pub(crate) struct AesWorker {
     flags: Flags,
     is_memfd: bool,
     should_exit: Arc<AtomicBool>,
+    transit_uids: Vec<(Uid, Uid)>,
+    transit_gids: Vec<(Gid, Gid)>,
 }
 
 impl AesWorker {
@@ -89,6 +88,8 @@ impl AesWorker {
         flags: Flags,
         is_memfd: bool,
         should_exit: Arc<AtomicBool>,
+        transit_uids: &[(Uid, Uid)],
+        transit_gids: &[(Gid, Gid)],
     ) -> Self {
         Self {
             fdalg,
@@ -96,6 +97,8 @@ impl AesWorker {
             flags,
             is_memfd,
             should_exit,
+            transit_uids: transit_uids.to_vec(),
+            transit_gids: transit_gids.to_vec(),
         }
     }
 
@@ -108,20 +111,21 @@ impl AesWorker {
                 // Honour dry-run when exporting.
                 let dry_run =
                     env::var_os(ENV_SKIP_SCMP).is_some() || ExportMode::from_env().is_some();
-                if !dry_run {
-                    // SAFETY: Default panic hook wont play well with seccomp.
-                    std::panic::set_hook(Box::new(|_| {}));
-                }
 
-                let ctx = Self::prepare_confine(self.flags)?;
                 if !dry_run {
-                    // SAFETY: Logging will kill us after seccomp.
+                    let ctx =
+                        Self::prepare_confine(self.flags, &self.transit_uids, &self.transit_gids)?;
+
                     let safe_setid = self
                         .flags
                         .intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID);
+
+                    // Log before load, log after will kill.
+                    // TODO: Ensure syd_aes threads can write to log fd.
                     info!("ctx": "confine", "op": "confine_aes_thread",
                         "msg": format!("AES thread confined with{} SROP mitigation",
                             if safe_setid { "out" } else { "" }));
+
                     ctx.load()?;
                 } else {
                     error!("ctx": "confine", "op": "confine_aes_thread",
@@ -271,7 +275,7 @@ impl AesWorker {
         #[allow(clippy::cast_sign_loss)]
         let data_size = lseek64(&crypt_fd, 0, Whence::SeekEnd).unwrap() as u64;
         if data_size == 0 {
-            ftruncate64(enc_fd.as_raw_fd(), 0).unwrap();
+            retry_on_eintr(|| ftruncate64(&enc_fd, 0)).unwrap();
             return Ok(());
         }
 
@@ -320,9 +324,7 @@ impl AesWorker {
             if last_block_offset < file_size {
                 // Truncate the encrypted file to remove the partial block.
                 let truncate_offset = header_size + last_block_offset;
-                retry_on_eintr(|| {
-                    ftruncate64(enc_fd.as_raw_fd(), truncate_offset.try_into().unwrap())
-                })?;
+                retry_on_eintr(|| ftruncate64(&enc_fd, truncate_offset.try_into().unwrap()))?;
 
                 // Adjust crypt_fd to read from the last full block offset.
                 #[allow(clippy::cast_possible_wrap)]
@@ -380,7 +382,7 @@ impl AesWorker {
             if file_size > 0 {
                 // Remove previous content,
                 // SAFETY: wipe IV to avoid reuse.
-                retry_on_eintr(|| ftruncate64(enc_fd.as_raw_fd(), 0))?;
+                retry_on_eintr(|| ftruncate64(&enc_fd, 0))?;
                 lseek64(&enc_fd, 0, Whence::SeekSet)?;
             }
 
@@ -642,7 +644,7 @@ impl AesWorker {
         lseek64(&enc_fd, CRYPT_MAGIC.len() as i64, Whence::SeekSet)?;
 
         // Write the HMAC tag to the file.
-        let buf = hmac_tag.unsecure();
+        let buf = hmac_tag.as_slice();
         let mut nwrite = 0;
         while nwrite < buf.len() {
             #[allow(clippy::arithmetic_side_effects)]
@@ -660,7 +662,11 @@ impl AesWorker {
 
     /// Confine AES thread.
     #[allow(clippy::cognitive_complexity)]
-    pub(crate) fn prepare_confine(flags: Flags) -> SydResult<ScmpFilterContext> {
+    pub(crate) fn prepare_confine(
+        flags: Flags,
+        transit_uids: &[(Uid, Uid)],
+        transit_gids: &[(Gid, Gid)],
+    ) -> SydResult<ScmpFilterContext> {
         // Create seccomp filter with default action.
         let mut ctx = ScmpFilterContext::new(ScmpAction::KillProcess)?;
 
@@ -670,7 +676,7 @@ impl AesWorker {
 
         // Disable Speculative Store Bypass mitigations
         // with trace/allow_unsafe_spec_exec:1
-        ctx.set_ctl_ssb(flags.contains(Flags::FL_ALLOW_UNSAFE_SPEC_EXEC))?;
+        ctx.set_ctl_ssb(flags.allow_unsafe_spec_exec())?;
 
         // DO NOT synchronize filter to all threads.
         // Main thread will confine itself.
@@ -705,28 +711,49 @@ impl AesWorker {
                     ctx.add_rule(ScmpAction::Errno(Errno::ENOSYS as i32), syscall)?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_aes_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
         }
 
-        // Allow reads/writes up to HMAC & IV size.
+        // Allow reads up to MAX(HMAC | IV) bytes.
         // See the note in sync_file().
-        for sysname in ["read", "write"] {
-            match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => {
-                    ctx.add_rule_conditional(
-                        ScmpAction::Allow,
-                        syscall,
-                        &[scmp_cmp!($arg2 <= HMAC_TAG_SIZE.max(IV_SIZE) as u64)],
-                    )?;
-                }
-                Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
-                }
+        let rw_max = u64::try_from(HMAC_TAG_SIZE.max(IV_SIZE))?;
+        match ScmpSyscall::from_name("read") {
+            Ok(syscall) => {
+                ctx.add_rule_conditional(
+                    ScmpAction::Allow,
+                    syscall,
+                    &[scmp_cmp!($arg2 <= rw_max)],
+                )?;
             }
+            Err(_) => {
+                info!("ctx": "confine", "op": "allow_aes_syscall",
+                    "msg": "invalid or unsupported syscall read");
+            }
+        }
+
+        // Allow writes to log-fd OR up to MAX(HMAC | IV) bytes.
+        // No proc_pid_mem(5) access required here.
+        confine_scmp_write(&mut ctx, Some(rw_max), false)?;
+
+        // Deny pipe2(2) O_NOTIFICATION_PIPE flag.
+        let sysname = "pipe2";
+        #[allow(clippy::cast_sign_loss)]
+        if let Ok(syscall) = ScmpSyscall::from_name(sysname) {
+            // O_NOTIFICATION_PIPE is equivalent to O_EXCL,
+            // see: linux/watch_queue.h
+            const O_NOTIFICATION_PIPE: u64 = OFlag::O_EXCL.bits() as u64;
+
+            ctx.add_rule_conditional(
+                ScmpAction::Allow,
+                syscall,
+                &[scmp_cmp!($arg1 & O_NOTIFICATION_PIPE == 0)],
+            )?;
+        } else {
+            info!("ctx": "confine", "op": "allow_aes_syscall",
+                "msg": format!("invalid or unsupported syscall {sysname}"));
         }
 
         // Allow safe fcntl(2) utility calls.
@@ -734,7 +761,7 @@ impl AesWorker {
             let syscall = match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => syscall,
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_aes_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                     continue;
                 }
@@ -768,7 +795,7 @@ impl AesWorker {
                     ctx.add_rule(ScmpAction::Allow, syscall)?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_aes_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
@@ -781,7 +808,7 @@ impl AesWorker {
                     ctx.add_rule(ScmpAction::Allow, syscall)?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_aes_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
@@ -794,42 +821,24 @@ impl AesWorker {
                     ctx.add_rule(ScmpAction::Allow, syscall)?;
                 }
                 Err(_) => {
-                    info!("ctx": "confine", "op": "allow_syscall",
+                    info!("ctx": "confine", "op": "allow_aes_syscall",
                         "msg": format!("invalid or unsupported syscall {sysname}"));
                 }
             }
         }
 
-        if flags.intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID) {
-            // SAFETY: Main thread confines these further.
-            // As these system calls as per-process,
-            // the main thread's seccomp rules will apply
-            // to us even without TSYNC.
-            for sysname in SET_ID_SYSCALLS {
-                match ScmpSyscall::from_name(sysname) {
-                    Ok(syscall) => {
-                        ctx.add_rule(ScmpAction::Allow, syscall)?;
-                    }
-                    Err(_) => {
-                        info!("ctx": "confine", "op": "allow_syscall",
-                            "msg": format!("invalid or unsupported syscall {sysname}"));
-                    }
-                }
-            }
-
-            // SAFETY:
-            // Signal system calls are necessary to handle reserved signals.
-            for sysname in ["sigreturn", "rt_sigreturn"] {
-                match ScmpSyscall::from_name(sysname) {
-                    Ok(syscall) => {
-                        ctx.add_rule(ScmpAction::Allow, syscall)?;
-                    }
-                    Err(_) => {
-                        info!("ctx": "confine", "op": "allow_syscall",
-                            "msg": format!("invalid or unsupported syscall {sysname}"));
-                    }
-                }
-            }
+        // Allow UID/GID changing system calls as necessary.
+        let safe_setuid = flags.allow_safe_setuid();
+        let safe_setgid = flags.allow_safe_setgid();
+        if safe_setuid || safe_setgid {
+            scmp_add_setid_rules(
+                "aes",
+                &mut ctx,
+                safe_setuid,
+                safe_setgid,
+                transit_uids,
+                transit_gids,
+            )?;
         }
 
         Ok(ctx)
