@@ -56,7 +56,6 @@ use crate::{
         TRUNCATE_COOKIE_ARG4, TRUNCATE_COOKIE_ARG5, UNLINKAT_COOKIE_ARG3, UNLINKAT_COOKIE_ARG4,
         UNLINKAT_COOKIE_ARG5,
     },
-    crit,
     err::{err2no, SydJoinHandle, SydResult},
     fs::{
         seccomp_notify_respond, AT_EXECVE_CHECK, SECCOMP_IOCTL_NOTIF_ADDFD,
@@ -135,25 +134,6 @@ impl Worker {
             .name("syd_emu".to_string())
             .stack_size(EMU_STACK_SIZE)
             .spawn(move || {
-                // Set a logging panic hook. The default panic
-                // hook calls system calls not permitted by emulators
-                // such as getcwd(2), stat(2) etc.
-                std::panic::set_hook(Box::new(|info| {
-                    let err = match info.payload().downcast_ref::<&'static str>() {
-                        Some(s) => *s,
-                        None => match info.payload().downcast_ref::<String>() {
-                            Some(s) => &**s,
-                            None => "?",
-                        },
-                    };
-
-                    let file = info.location().map(|l| l.file());
-                    let line = info.location().map(|l| l.line());
-
-                    crit!("ctx": "panic", "op": "panic_emulator_thread",
-                        "msg": err, "file": file, "line": line);
-                }));
-
                 // Unshare:
                 // 1. CLONE_FS so cwd and umask are per-thread.
                 // 2. CLONE_FILES so file descriptor table is per-thread.
