@@ -43,8 +43,7 @@ use crate::{
     alert,
     compat::{epoll_ctl_mod_safe, epoll_ctl_safe},
     config::*,
-    confine::scmp_add_setid_rules,
-    crit,
+    confine::{confine_scmp_write, scmp_add_setid_rules},
     err::{err2no, scmp2no, SydJoinHandle, SydResult},
     fs::{closeexcept, duprand, peer_cred, retry_on_eintr},
     hash::SydHashMap,
@@ -471,33 +470,9 @@ impl IpcWorker {
             }
         }
 
-        // SAFETY: Allow write(2) to syd::log::LOG_FD only.
-        // If logging is disabled, deny write(2) completely.
-        // Note, we do allow the use-case where LOG_FD is
-        // intentionally set to a negative value to disable
-        // logging, hence we cast directly to an u64 and not
-        // use u64::try_from!
-        // Exception: Allow write(2) globally if profiling is enabled.
-        let log_fd = crate::log::LOG_FD.load(Ordering::Relaxed);
-        for sysname in ["write", "writev"] {
-            #[allow(clippy::cast_sign_loss)]
-            match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) if cfg!(feature = "prof") => {
-                    ctx.add_rule(ScmpAction::Allow, syscall)?;
-                }
-                Ok(syscall) => {
-                    ctx.add_rule_conditional(
-                        ScmpAction::Allow,
-                        syscall,
-                        &[scmp_cmp!($arg0 == log_fd as u64)],
-                    )?;
-                }
-                Err(_) => {
-                    info!("ctx": "confine", "op": "allow_ipc_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
-                }
-            }
-        }
+        // Allow writes to the log-fd.
+        // No proc_pid_mem(5) access required here.
+        confine_scmp_write(&mut ctx, None, false)?;
 
         // Allow safe system calls.
         for sysname in IPC_SYSCALLS {
@@ -569,23 +544,6 @@ impl IpcWorker {
 
     /// Initialize IPC worker.
     fn init(&mut self, notif_pipe: (RawFd, RawFd)) -> SydResult<()> {
-        // SAFETY: Default panic hook wont work with stdio(3) closed.
-        std::panic::set_hook(Box::new(|info| {
-            let err = match info.payload().downcast_ref::<&'static str>() {
-                Some(s) => *s,
-                None => match info.payload().downcast_ref::<String>() {
-                    Some(s) => &**s,
-                    None => "?",
-                },
-            };
-
-            let file = info.location().map(|l| l.file());
-            let line = info.location().map(|l| l.line());
-
-            crit!("ctx": "panic", "op": "panic_ipc_thread",
-                "msg": err, "file": file, "line": line);
-        }));
-
         // SAFETY: We use exit_group(2) here to bail,
         // because this unsharing is a critical safety feature.
         if let Err(errno) = unshare(CloneFlags::CLONE_FS | CloneFlags::CLONE_FILES) {
